// Declaring a namespace for the application.
var APP = APP || {};

/**********************************************************
 * Application execution                                  *
 **********************************************************/

function onGeomajasLoad() {

	// Future rest api data?
	// http://reisinfo.delijn.be/rise-api-web/haltes/vertrekken/202359/13

	var loading =
		"<p>Updating</p>" +
		"<p>location and routes</p>" +
		"<img src='img/ajax-loader.gif' width='100px' height='100px' />";

	var stopList = document.getElementById("stopList");
	stopList.innerHTML = loading;

	// Load and parse JSON file.
	// This should be called only once.
	APP.Common.loadData();

	// Set debug mode (enables logging)
	APP.Common.setDebug(false);
	// Set logging to console or servletLogging
	APP.Common.setLogToConsole(false);

	APP.Common.log("");
	APP.Common.log("- APP STARTING ...");
	APP.Common.log("");

	// Disable  for Jquery Mobile
	$(document).on("pageinit", function () {
		$.mobile.buttonMarkup.hoverDelay = 0;
		$.mobile.defaultPageTransition = 'none';
		$.mobile.defaultDialogTransition = 'none';
	});

	// Set the iframe and pre-load the contents.
	var busStopIframe = document.getElementById("busStopIframe");
	busStopIframe.contentWindow.location.replace("loading.html");

	// Set the correct width and height.
	busStopIframe.height = window.innerHeight - 36 + "px";
	APP.Common.log("HEIGHT: " + busStopIframe.height);
	busStopIframe.width = window.innerWidth + "px";
	APP.Common.log("WIDTH: " + busStopIframe.width);

	// Get the location first.
	APP.Geolocation.updateLocation();

	APP.OsmMap.initOsmMap();

	// Add jquery handlers
	$(document).on('pageshow','#home-page',function() {
		busStopIframe.contentWindow.location.replace("loading.html");
	});

	$(document).on('pageshow','#geomajas-map',function() {
		busStopIframe.contentWindow.location.replace("loading.html");
		APP.OsmMap.resizeMap();
		APP.OsmMap.goToCurrentLocation();
	});

	$('#zoomInButton').click(function() {
		APP.OsmMap.zoomIn();
	});

	$('#zoomOutButton').click(function() {
		APP.OsmMap.zoomOut();
	});

	$( window ).on( "orientationchange", function( event ) {
		setTimeout(function(){
			APP.OsmMap.resizeMap();
			busStopIframe.height = window.innerHeight - 36 + "px";
			APP.Common.log("HEIGHT: " + busStopIframe.height);
			busStopIframe.width = window.innerWidth + "px";
			APP.Common.log("WIDTH: " + busStopIframe.width);
		}, 500);
	});

	//////////////////////////////////////////
	// Pull down update action
	//////////////////////////////////////////

	// Item to which we attach the listener
	var pullItem = document.getElementById("pullHeader");
	//grab the list
	var contentList = document.getElementById("stopList");
	//keep the state whether the fingers are touched
	var isTouched = false;
	//keep the state whether a PULL actually went out
	var isMoved = false;
	//This has the original Top offset (relative to screen) position of the list
	var prevY = parseInt(pullItem.offsetTop);
	//This has the original Top CSS position of the list
	var cssY = pullItem.style.top;
	cssY = parseInt(cssY.substring(0,cssY.length - 2));

	//Add the start of the touching
	pullItem.addEventListener("touchstart",function(e){
		APP.Common.log("Pull touchestart");
		isTouched = true;
		//initialize the touched point
		prevY = e.changedTouches[0].clientY;
		//we use css3 transitions when available for smooth slizading
		pullItem.style.transition = "";
		e.preventDefault();
	},false);
	pullItem.addEventListener("touchend",function(e){
		APP.Common.log("Pull touchend");
		//on touchup we cancel the touch event
		isTouched = false;
		//now if the list has moved downwards, it should come up but in a transition
		pullItem.style.transition = "top 1s";
		if(isMoved){
			loadNewData();
		}
		pullItem.style.top = cssY + 'px';
		isMoved = false;

		e.preventDefault();
	},false);
	pullItem.addEventListener("touchmove",function(e){
		APP.Common.log("Pull touchemove");
		if(isTouched){
			if(e.changedTouches[0].clientY > prevY){
				//on touchmove, we add the exact amount fingers moved to the top of the list
				var change = e.changedTouches[0].clientY - prevY;
				//and add it to the style
				pullItem.style.top = cssY + change + 'px';
				isMoved = true;
			}
		}
		e.preventDefault();
	},false);


	//binding mouse events to make this work in desktop browsers as well
	pullItem.addEventListener("mousedown",function(e){
		APP.Common.log("Pull Mousedown");
		isTouched = true;
		prevY = e.clientY;
		pullItem.style.transition = "";
		e.preventDefault();
	},false);
	pullItem.addEventListener("mouseup",function(e){
		APP.Common.log("Pull Mouseup");
		isTouched = false;

		pullItem.style.transition = "top 1s";
		if(isMoved){
			loadNewData();
		}
		pullItem.style.top = cssY + 'px';
		isMoved = false;

		e.preventDefault();
	},false);
	pullItem.addEventListener("mousemove",function(e){
		APP.Common.log("Pull Mousemove");
		if(isTouched){
			if(e.clientY > prevY){
				var change = e.clientY - prevY;
				pullItem.style.top = cssY + change + 'px';
				isMoved = true;
			}
		}
		e.preventDefault();
	},false);

	function loadNewData(){
		var stopList = document.getElementById("stopList");
		stopList.innerHTML = loading;
		APP.Geolocation.updateLocation();
		setTimeout(function(){
			//APP.Common.updateBusStops();
			APP.Common.loadData();
		}, 2000);

	}

}

/**********************************************************
 * Common properties and methods fore the application.    *
 **********************************************************/
APP.Common = (function () {
	'use strict';

	/**********************
	 * PRIVATE PROPERTIES *
	 **********************/
	var busStops            = [],
		closestBusStops     = [],
		showMaxStops        = 20,
		kdTreeObject        = null,
		originalCoordinate  = null,
		currentCoordinate   = null,
		logToConsole        = true,
		debug               = false,
		SERVICE_URL          = "http://reisinfo.delijn.be/realtime/halte/";
		/*SERVICE_URL          = "http://www.delijn.be/en/haltes/halte/";*/

	/**********************
	 * PRIVATE METHODS    *
	 **********************/
	function log(text) {
		if (debug) {
			if (logToConsole) {
				console.log(text);
			} else {
				$.get("log/?log=" + text, function (data) {
				});
			}
		}
	}

	function loadJSON(file, callback) {
		var xobj = new XMLHttpRequest();
		xobj.overrideMimeType("application/json");
		xobj.open('GET', file, true);
		xobj.onreadystatechange = function () {
			if (xobj.readyState == 4 && xobj.status == "200") {
				callback(xobj.responseText);
			}
		};
		xobj.send(null);
	}

	function loadDataObject() {

		if (APP.Geolocation.getX() != null) {

			// Retrieve the correct Json file based on location and bounds.
			loadJSON("data/configuration.json", function (response) {

				log("Loading configuration ...");

				var parsedJSON = JSON.parse(response);

				function getDistanceBetween(a, b) {
					var dx = a.x - b.x;
					var dy = a.y - b.y;
					return Math.sqrt(dx*dx + dy*dy);
				}

				busStops = [];

				for ( var i = 0; i < parsedJSON.length; i++) {

					if (APP.Toolbox.containsPosition(parsedJSON[i].bounds)) {

						var files = [];
						files.push(parsedJSON[i].fileName);
						//console.log("ORIGINAL: " + parsedJSON[i].fileName);

						var include = APP.Toolbox.getBorder(parsedJSON[i].bounds);
						//console.log(include);

						kdTreeObject = null;
						kdTreeObject = new kdTree([], getDistanceBetween, ["x", "y"]);

						(function () {

							for ( var x = 0; x < parsedJSON.length; x++) {

								var original = parsedJSON[i].bounds;
								var compare = parsedJSON[x].bounds;

								switch (include) {
									// NONE
									case 0:
										// exit the function and only show the original file.
										return;
									// TOP
									case 1:
										if (
											compare.minX == original.minX &&
											compare.minY == original.maxY
											) {
											//console.log("TOP: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// RIGHT
									case 2:
										if (
											compare.minX == original.maxX &&
											compare.minY == original.minY
											) {
											//console.log("RIGHT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// BOTTOM
									case 3:
										if (
											compare.minX == original.minX &&
											compare.maxY == original.minY
											) {
											//console.log("BOTTOM: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// LEFT
									case 4:
										if (
											compare.maxX == original.minX &&
											compare.minY == original.minY
											) {
											//console.log("LEFT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// TOP_RIGHT
									case 5:
										if (
											compare.minX == original.minX &&
											compare.minY == original.maxY
											) {
											//console.log("TOP: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.minX == original.maxX &&
											compare.minY == original.minY
											) {
											//console.log("RIGHT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.minX == original.maxX &&
											compare.minY == original.maxY
											) {
											//console.log("TOP-RIGHT-CORNER: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// BOTTOM_RIGHT
									case 6:
										if (
											compare.minX == original.minX &&
											compare.maxY == original.minY
											) {
											//console.log("BOTTOM: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.minX == original.maxX &&
											compare.minY == original.minY
											) {
											//console.log("RIGHT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.minX == original.maxX &&
											compare.maxY == original.minY
											) {
											//console.log("BOTTOM-RIGHT-CORNER: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// BOTTOM LEFT
									case 7:
										if (
											compare.minX == original.minX &&
											compare.maxY == original.minY
											) {
											//console.log("BOTTOM: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.maxX == original.minX &&
											compare.minY == original.minY
											) {
											//console.log("LEFT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.maxX == original.minX &&
											compare.maxY == original.minY
											) {
											//console.log("BOTTOM-LEFT-CORNER: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
									// TOP_LEFT
									case 8:
										if (
											compare.minX == original.minX &&
											compare.minY == original.maxY
											) {
											//console.log("TOP: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.maxX == original.minX &&
											compare.minY == original.minY
											) {
											//console.log("LEFT: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										if (
											compare.maxX == original.minX &&
											compare.minY == original.maxY
											) {
											//console.log("TOP-LEFT-CORNER: " + parsedJSON[x].fileName);
											files.push(parsedJSON[x].fileName);
										}
										break;
								}

							}

						}());

						//console.log("FILES FOUND: " + files.length);
						for (var t = 0; t < files.length; t++) {

							// Load
							loadJSON("data/" + files[t], function (response) {

								log("Loading Json ...");

								var parsedJSON = JSON.parse(response);

								//console.log("Total of bus stops: " + parsedJSON.busStops.length);

								function getDistanceBetween(a, b) {
									var dx = a.x - b.x;
									var dy = a.y - b.y;
									return Math.sqrt(dx*dx + dy*dy);
								}

								for ( var i = 0; i < parsedJSON.busStops.length; i++) {

									// Create a new object.
									var BusStop = {
										id: parsedJSON.busStops[i].id,
										name: parsedJSON.busStops[i].name,
										x: parsedJSON.busStops[i].x,
										y: parsedJSON.busStops[i].y
									};

									kdTreeObject.insert({
										x: BusStop.x,
										y: BusStop.y
									});

									// Add it to our list.
									busStops.push(BusStop);

								}

								log("Json loaded.");

							});

						}

						setTimeout(function(){
							updateClosestBusStops();
						}, 1000);

						break;

					}

				}

			});

		} else {
			log("No location yet.");

			setTimeout(function(){
				loadDataObject();
			}, 100);
		}

	}

	function addListeners() {
		for (var i = 0; i < closestBusStops.length; i++) {

			var stopListItem = document.getElementById("stopListItem-" + i);

			stopListItem.addEventListener("mousedown",function(event) {
				event.stopPropagation();
				APP.Common.log("mouseDownOnList: " + event.clientY);
				originalCoordinate = event.clientY;
			}, false);

			stopListItem.addEventListener("mouseup",function(event) {
				event.stopPropagation();
				APP.Common.log("mouseUpOnList: " + event.clientY);
				currentCoordinate = event.clientY;

				if (originalCoordinate == currentCoordinate) {

					APP.Common.log("Loading external data ...");

					try {

						var busStopIframe = document.getElementById("busStopIframe");
						busStopIframe.contentWindow.location.replace(event.currentTarget.title);

						$(':mobile-pagecontainer').pagecontainer('change', '#link-page', {
						});

						APP.Common.log(event.currentTarget.title + " loading ...");

					}
					catch (err) {
						alert(err.message);
					}

				}

			}, false);

			stopListItem.addEventListener('touchstart',function(event) {
				event.stopPropagation();
				APP.Common.log("touchStartOnList: " + event.changedTouches[0].clientY);
				originalCoordinate = event.changedTouches[0].clientY;
			}, false);

			stopListItem.addEventListener('touchmove',function(event) {
				event.stopPropagation();
				APP.Common.log("touchMoveOnList: " + event.changedTouches[0].clientY);
				currentCoordinate = event.changedTouches[0].clientY;
			}, false);

			stopListItem.addEventListener('touchend',function(event) {
				event.stopPropagation();
				APP.Common.log("touchEndOnList: " + event.changedTouches[0].clientY);
				currentCoordinate = event.changedTouches[0].clientY;
			}, false);

		}
	}

	function updateClosestBusStops() {

		function checkEqualCoords(value) {
			for (var i = 0; i < closestStops.length; i++) {
				if (value.x == closestStops[i][0].x && value.y == closestStops[i][0].y) {
					value.distance = closestStops[i][1];
					return true;
				}
			}
			return false;
		}

		if (kdTreeObject != null && APP.Geolocation.getX() != null) {

			closestBusStops = [];
			var stopList = document.getElementById("stopList");
			stopList.innerHTML = "";

			log("Updating bus stops ...");

			if (busStops.length < showMaxStops) {
				showMaxStops = busStops.length;
			}

			var closestStops = kdTreeObject.nearest(
				{
					x: APP.Geolocation.getX(),
					y: APP.Geolocation.getY()
				},
				showMaxStops
			);

			closestBusStops = busStops.filter(checkEqualCoords);
			closestBusStops.sort(function(a, b) {return a['distance'] - b['distance']});

			// Update our dom structure.
			for (var i = 0; i < closestBusStops.length; i++) {

				var stopId = closestBusStops[i].id;
				var stopName = closestBusStops[i].name;
				var stopDistance = Math.round(closestBusStops[i].distance);
				var stopUrl = APP.Common.getServiceUrl() + closestBusStops[i].id;

				stopList.innerHTML = stopList.innerHTML +
					"<li class='stopItem'>" +
					"<a id='stopListItem-" + i + "' title='" + stopUrl + "'>" +
					"<span class='stopInfoTop'>" +
					"<span class='left icon-flag'>&nbsp;" + stopId + "</span>" +
					"<span class='right icon-road'>&nbsp;" + stopDistance + "m</span>" +
					"</span>" +
					"<span class='stopInfoBottom'>" + stopName + "</span>" +
					"</a>" +
					"</li>"
				;

			}

			addListeners();

			log("Bus stops updated.");

			APP.OsmMap.addMarkers();

		} else {
			log("KDTREE not filled yet.");

			setTimeout(function(){
				updateClosestBusStops();
			}, 100);
		}

	}

	/**********************
	 * PUBLIC METHODS     *
	 **********************/
	return {
		setLogToConsole: function (toConsole) { logToConsole = toConsole; },
		setDebug: function (isDebug) { debug = isDebug; },
		getClosestBusStops: function() { return closestBusStops; },
		getServiceUrl: function() { return SERVICE_URL; },
		loadData: loadDataObject,
		updateBusStops: updateClosestBusStops,
		log: log
	};

}());

/**********************************************************
 * Geolocation specific methods and parameters.           *
 **********************************************************/
APP.Geolocation = (function () {
	'use strict';

	/**********************
	 * PRIVATE PROPERTIES *
	 **********************/
	var currentX            = null,
		currentY            = null,
		watchId             = null,
		locationOptions     = {enableHighAccuracy: true, timeout: 30000, maximumAge: 2000};

	/**********************
	 * PRIVATE METHODS    *
	 **********************/
	function updateLocation() {
		if(navigator.geolocation) {

			APP.Common.log("Updating location ...");

			watchId = navigator.geolocation.watchPosition(
				setLocation,
				showError,
				locationOptions
			);
		} else {
			alert("Geolocation not available.");
		}
	}

	function stopWatch () {
		if (watchId != null) {
			navigator.geolocation.clearWatch(watchId);
			watchId = null;
		}
	}

	function setLocation(pos) {

		APP.Common.log("Accuracy: " + pos.coords.accuracy);
		if (pos.coords.accuracy <= 20) {
			stopWatch();
			APP.Common.updateBusStops();
		}

		var currentCoordinate = APP.Toolbox.EPSG4326ToEPSG31370(
			pos.coords.longitude,
			pos.coords.latitude
		);
		currentX = currentCoordinate[0];
		currentY = currentCoordinate[1];

		APP.Common.log("Location updated to: [ x: " + currentX + ", y: " + currentY  + " ]");
	}

	function showError(error) {
		switch(error.code) {
			case error.PERMISSION_DENIED:
				alert("User denied the request for Geolocation.");
				break;
			case error.POSITION_UNAVAILABLE:
				alert("Location information is unavailable.");
				break;
			case error.TIMEOUT:
				//alert("The request to get user location timed out.");
				stopWatch();
				break;
			case error.UNKNOWN_ERROR:
				alert("An unknown error occurred.");
				break;
		}
	}

	/**********************
	 * PUBLIC METHODS     *
	 **********************/
	return {
		setX: function (corX) { currentX = corX; },
		getX: function () { return currentX; },
		setY: function (corY) { currentY = corY; },
		getY: function () { return currentY; },
		setWatchId: function (id) { watchId = id; },
		getWatchId: function () { return watchId; },
		updateLocation: updateLocation
	};

}());

/**********************************************************
 * Map related method and properties.                     *
 **********************************************************/
APP.OsmMap = (function () {
	'use strict';

	/**********************
	 * PRIVATE PROPERTIES *
	 **********************/
	var OSM_BASE_URL            = "http://a.tile.openstreetmap.org/{z}/{x}/{y}.png",
		CRS                     = "EPSG:900913",
		MAX_ZOOM_LEVELS         = 20,
		EQUATOR_IN_METERS       = 40075016.686,
		HALF_EQUATOR_IN_METERS  = 40075016.686 / 2,
		map                     = null,
		resolutions             = [],
		markerContainer         = null;

	/**********************
	 * PRIVATE METHODS    *
	 **********************/
	function initMap() {
		APP.Common.log("Generating resolutions ...");

		for (var i = 0; i < MAX_ZOOM_LEVELS; i++) {
			resolutions.push(EQUATOR_IN_METERS / (256 * Math.pow(2, i)));
		}

		APP.Common.log("Resolutions generated.");

		showStopMap();
	}

	function resizeMap() {

		APP.Common.log("Resizing map ...");

		if (map != null && $("#geomajas-map").is(':visible') ) {
			var wHeight = window.innerHeight ? window.innerHeight : $(window).height();
			var mapHeight = wHeight - ($('#geomajas-map-header').height() * 2) - 5;
			var mapWidth =  $('#geomajas-map').width();
			map.setSize(mapWidth, mapHeight);
		}

		APP.Common.log("Map has been resized.");

	}

	function showMarkers() {
		if (map == null) {
			return;
		}

		APP.Common.log("Adding markers to the map ...");

		if (markerContainer === null) {
			markerContainer = map.getContainerManager().addWorldWidgetContainer();
		} else {
			markerContainer.clear();
		}

		var mapElement = document.getElementById(map.getParentHtmlElementId());
		var rect = mapElement.getBoundingClientRect();
		var originalCoordinate = null;
		var currentCoordinate = null;

		// Add a current position marker to the map
		var currentPositionDom = document.createElement("img");
		currentPositionDom.setAttribute('src', 'img/current.png');
		var currentMarker = new gm.Marker(currentPositionDom, 24, 48);
		markerContainer.addMarker(currentMarker);
		var currentPositionCor = APP.Toolbox.EPSG31370ToEPSG900913(
			APP.Geolocation.getX(),
			APP.Geolocation.getY()
		);
		var currentPositionCoordinate = new gm.Coordinate(currentPositionCor[0], currentPositionCor[1]);
		currentMarker.setCoordinate(currentPositionCoordinate);

		// Add all bus stop markers to the map.
		for (var i = 0; i < APP.Common.getClosestBusStops().length; i++) {

			//marker image element
			var markerDom = document.createElement("img");
			markerDom.setAttribute('id', 'marker-' + i);
			markerDom.setAttribute('src', 'img/busStop.png');
			markerDom.setAttribute('data-url', APP.Common.getServiceUrl() + APP.Common.getClosestBusStops()[i].id/*, '_system'*/);
			// Create new marker with the coordinates:
			// (the image is 32 x 32 so we center the mouse in the middle)
			var stopMarker = new gm.Marker(markerDom, 16, 16);
			// Don't forget to add the marker to the container so it will be drawn:
			markerContainer.addMarker(stopMarker);

			//set current coordinate
			var current = APP.Toolbox.EPSG31370ToEPSG900913(
				APP.Common.getClosestBusStops()[i].x,
				APP.Common.getClosestBusStops()[i].y
			);

			APP.Common.log(JSON.stringify(APP.Common.getClosestBusStops()[i]));

			var markerCoordinate = new gm.Coordinate(current[0], current[1]);

			stopMarker.setCoordinate(markerCoordinate);

			// Add native listeners to our dom element.
			var busStop = document.getElementById("marker-" + i);

			busStop.addEventListener("mousedown", function (event) {
				event.stopPropagation();
				APP.Common.log("mouseDownOnMap: " + event.clientY);
				originalCoordinate = event.clientY;
			}, false);

			busStop.addEventListener("mouseup", function (event) {
				event.stopPropagation();
				APP.Common.log("mouseUpOnMap: " + event.clientY);
				currentCoordinate = event.clientY;

				if (originalCoordinate == currentCoordinate) {

					var busStopIframe = document.getElementById("busStopIframe");
					busStopIframe.contentWindow.location.replace(event.srcElement.getAttribute('data-url'));

					$(':mobile-pagecontainer').pagecontainer('change', '#link-page', {
					});

					APP.Common.log(event.srcElement.getAttribute('data-url') + " loading ...");

				}

			}, false);

			busStop.addEventListener('touchstart', function (event) {
				event.stopPropagation();
				APP.Common.log("touchStartOnMap: " + event.touches[0].screenY);
				originalCoordinate = event.touches[0].screenY;
			}, false);

			busStop.addEventListener('touchmove', function (event) {
				event.stopPropagation();
				APP.Common.log("touchMoveOnMap: " + event.touches[0].screenY);
				currentCoordinate = event.touches[0].screenY;
			}, false);

			busStop.addEventListener('touchend', function (event) {
				event.stopPropagation();
				APP.Common.log("touchEndOnMap: " + event.touches[0].screenY);
				currentCoordinate = event.touches[0].screenY;
			}, false);

		}

		APP.Common.log("Markers have been added to the map.");

	}

	function showStopMap() {
		if (map == null) {

			APP.Common.log("Creating map configuration ...");

			var mapConfig = new gm.MapConfiguration();
			mapConfig.setCrs(CRS, "METRIC");
			var bounds = new gm.Bbox(-HALF_EQUATOR_IN_METERS, -HALF_EQUATOR_IN_METERS, EQUATOR_IN_METERS, EQUATOR_IN_METERS);
			mapConfig.setInitBounds(bounds);
			mapConfig.setMaxBounds(bounds);
			mapConfig.setResolutions(resolutions);

			// Initialize the map with the mapConfiguration.
			map = new gm.Map("js-map-element", mapConfig);

			// Hide the controls.
			document.getElementsByClassName("gm-ZoomControl")[0].style.visibility = "hidden";
			document.getElementsByClassName("gm-ZoomToRectangleControl")[0].style.visibility = "hidden";
			document.getElementsByClassName("gm-ScalebarControl")[0].style.visibility = "hidden";

			APP.Common.log("Map configuration is set.");

			APP.Common.log("Setting OSM layer ...");

			var tileConfig = new gm.layer.TileConfiguration(256, 256, new gm.Coordinate(-HALF_EQUATOR_IN_METERS, -HALF_EQUATOR_IN_METERS));
			tileConfig.setResolutions(resolutions);
			// Now create an OSM layer and add it to the map:
			var osmLayer = new gm.OsmLayer("osmCountries", tileConfig, OSM_BASE_URL);
			map.getLayersModel().addLayer(osmLayer);
			map.getViewPort().applyResolution(map.getViewPort().getResolution(17));

			APP.Common.log("OSM Layer is set.");
		}
	}

	function applyCurrentPosition() {
		var currentCoordinate = APP.Toolbox.EPSG31370ToEPSG900913(
			APP.Geolocation.getX(),
			APP.Geolocation.getY()
		);
		var position = new gm.Coordinate(currentCoordinate[0], currentCoordinate[1]);
		map.getViewPort().applyPosition(position);
	}

	function zoomIn() {
		var resolution = map.getViewPort().getResolution();
		map.getViewPort().applyResolution(resolution / parseFloat(2));
	}

	function zoomOut() {
		var resolution = map.getViewPort().getResolution();
		map.getViewPort().applyResolution(resolution * parseFloat(2));
	}

	/**********************
	 * PUBLIC METHODS     *
	 **********************/
	return {
		initOsmMap: initMap,
		goToCurrentLocation: applyCurrentPosition,
		zoomIn: zoomIn,
		zoomOut: zoomOut,
		resizeMap: resizeMap,
		addMarkers: showMarkers
	};

}());

/**********************************************************
 * A toolbox with handy functions and properties.         *
 **********************************************************/
APP.Toolbox = (function () {
	'use strict';

	/**********************
	 * PRIVATE PROPERTIES *
	 **********************/
	var EPSG4326    = "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs",
		EPSG31370   = "+proj=lcc +lat_1=51.16666723333333 +lat_2=49.8333339 +lat_0=90 +lon_0=4.367486666666666 +x_0=150000.013 +y_0=5400088.438 +ellps=intl +towgs84=-106.869,52.2978,-103.724,0.3366,-0.457,1.8422,-1.2747 +units=m +no_defs",
		EPSG900913  = "+proj=merc +a=6378137 +b=6378137 +lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0 +units=m +nadgrids=@null +no_defs",
		POS         = {
			NONE: 0,
			TOP: 1,
			RIGHT: 2,
			BOTTOM: 3,
			LEFT: 4,
			TOP_RIGHT: 5,
			BOTTOM_RIGHT: 6,
			BOTTOM_LEFT: 7,
			TOP_LEFT: 8
		},
		// Buffer in meters
		buffer      = 1000;

	/**********************
	 * PRIVATE METHODS    *
	 **********************/
	function transformEPSG31370ToEPSG900913(x, y) {
		if (null != x && null != y) {
			var transformedProjection = proj4(
				EPSG31370,
				EPSG900913).forward([x, y]
			);
			return [transformedProjection[0], transformedProjection[1]];
		}
		return [0, 0];
	}

	function transformEPSG4326ToEPSG900913(x, y) {
		if (null != x && null != y) {
			var transformedProjection = proj4(
				EPSG4326,
				EPSG900913).forward([x, y]
			);
			return [transformedProjection[0], transformedProjection[1]];
		}
		return [0, 0];
	}

	function transformEPSG4326ToEPSG31370(x, y) {
		if (null != x && null != y) {
			var transformedProjection = proj4(
				EPSG4326,
				EPSG31370).forward([x, y]
			);
			return [transformedProjection[0], transformedProjection[1]];
		}
		return [0, 0];
	}

	function isPositionInBounds(bounds) {

		if (APP.Geolocation.getX() < bounds.minX) {
			return false;
		}
		if (APP.Geolocation.getY() < bounds.minY) {
			return false;
		}
		if (APP.Geolocation.getX() > bounds.maxX) {
			return false;
		}
		if (APP.Geolocation.getY() > bounds.maxY) {
			return false;
		}
		return true;
	}

	function isCloseTo(bounds) {

		var x = APP.Geolocation.getX();
		var y = APP.Geolocation.getY();

		APP.Common.log("X: " + x);
		APP.Common.log("Y: " + y);
		APP.Common.log("minX: " + bounds.minX);
		APP.Common.log("minY: " + bounds.minY);
		APP.Common.log("maxX: " + bounds.maxX);
		APP.Common.log("maxY: " + bounds.maxY);

		// CORNERS HAVE TO BE CHECKED FIRST.

		// Check if the position is in the top right corner.
		if (y > bounds.maxY - buffer && x > bounds.maxX - buffer) { return POS.TOP_RIGHT; }
		// Check if the position is in the bottom right corner.
		if (y < bounds.minY + buffer && x > bounds.maxX - buffer) { return POS.BOTTOM_RIGHT; }
		// Check if the position is in the bottom left corner.
		if (y < bounds.minY + buffer && x < bounds.minX + buffer) { return POS.BOTTOM_LEFT; }
		// Check if the position is in the top left corner.
		if (y > bounds.maxY - buffer && x < bounds.minX + buffer) { return POS.TOP_LEFT; }

		// Check if the position is near the top side
		if (y > bounds.maxY - buffer) { return POS.TOP; }
		// Check if the position is near the right side
		if (x > bounds.maxX - buffer) { return POS.RIGHT; }
		// Check if the position is near the bottom side
		if (y < bounds.minY + buffer) { return POS.BOTTOM; }
		// Check if the position is near the left side
		if (x < bounds.minX + buffer) { return POS.LEFT; }

		return POS.NONE;
	}

	/**********************
	 * PUBLIC METHODS     *
	 **********************/
	return {
		EPSG31370ToEPSG900913: transformEPSG31370ToEPSG900913,
		EPSG4326ToEPSG900913: transformEPSG4326ToEPSG900913,
		EPSG4326ToEPSG31370: transformEPSG4326ToEPSG31370,
		containsPosition: isPositionInBounds,
		getBorder: isCloseTo
	};

}());